/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Wei�enfelser Stra�e 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import com.bitctrl.resource.Configuration;
import com.bitctrl.util.logging.LoggerTools;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.lmstic3.MeldungenLoeschen;
import de.bsvrz.kex.lmstic3.empfang.dekoder.TIC3InfoDatei;
import de.bsvrz.kex.lmstic3.empfang.dekoder.TIC3InfoDateiDaten;
import de.bsvrz.kex.lmstic3.empfang.dekoder.TIC3InfoDateiException;
import de.bsvrz.kex.lmstic3.vew.daten.DavDatenModell;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Meldung;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3MeldungsDaten;

/**
 * Erzeugt eine RDS-Meldung im Datenverteiler.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class VersendeMeldungen implements ClientSenderInterface {
	/** Der Logger der Klassse. */
	private static Logger log = Logger.getLogger(MeldungenLoeschen.class.getName());

	private static ClientDavConnection verbindung;

	private static Configuration konfig;

	/**
	 * @throws Exception
	 */
	@BeforeClass
	public static void beforeClass() throws Exception {

		final Level level;

		VersendeMeldungen.konfig = Configuration.getConfiguration();
		level = Level.parse(VersendeMeldungen.konfig.getString("debug.level"));
		LoggerTools.setLoggerLevel(level);
		VersendeMeldungen.log.setLevel(level);

		VersendeMeldungen.log.info("Stelle Verbindung zum Datenverteiler her ...");
		VersendeMeldungen.verbindung = new ClientDavConnection();
		VersendeMeldungen.verbindung.getClientDavParameters()
				.setDavCommunicationAddress(VersendeMeldungen.konfig.getString("dav.host"));
		VersendeMeldungen.verbindung.getClientDavParameters()
				.setDavCommunicationSubAddress(VersendeMeldungen.konfig.getInt("dav.port"));
		VersendeMeldungen.verbindung.connect();
		VersendeMeldungen.verbindung.login(VersendeMeldungen.konfig.getString("dav.benutzer"),
				VersendeMeldungen.konfig.getString("dav.kennwort"));

	}

	/**
	 * Prefix der PID der zu erzeugenden Meldung.
	 */
	private final String meldungsPidPrefix = "rdsmeldungtest.";

	/**
	 * Landesmeldestelle.
	 */
	private final String landesmeldeStelle = "lmst.bw";

	private List<TIC3Meldung> bildeMeldungenAusDatei(final String dateiname)
			throws TIC3InfoDateiException, TIC3Exception {
		final TIC3InfoDatei tid = new TIC3InfoDatei(new File(dateiname));
		final List<TIC3Meldung> meldungen = new ArrayList<>();

		final TIC3InfoDateiDaten bildeMeldungen = tid.bildeMeldungen(false);

		for (final TIC3MeldungsDaten meldungsdaten : bildeMeldungen.getMeldungen()) {
			meldungen.add(neueMeldung(meldungsdaten));
		}
		return meldungen;
	}

	private List<TIC3Meldung> bildeMeldungenAusDateien() throws TIC3InfoDateiException, TIC3Exception {
		final List<TIC3Meldung> meldungen = new ArrayList<>();
		final String dateiliste = VersendeMeldungen.konfig.getString("VersandMeldungen");

		for (final String datei : dateiliste.split(",")) {
			meldungen.addAll(bildeMeldungenAusDatei(datei));
		}

		return meldungen;
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		// TODO Auto-generated method stub

	}

	/**
	 * Erzeugt eine RDS-Meldung und versendet diese an den DaV.
	 */
	@Test
	public void erzeugeMeldungen() {

		final SystemObject lms = VersendeMeldungen.verbindung.getDataModel().getObject(landesmeldeStelle);

		try {
			final DataDescription dd = new DataDescription(
					VersendeMeldungen.verbindung.getDataModel().getAttributeGroup("atg.rdsMeldung"),
					VersendeMeldungen.verbindung.getDataModel().getAspect("asp.rdsSenden"));

			DavDatenModell.getInstanz().setVerbindung(VersendeMeldungen.verbindung);

			final MutableSet meldungsmenge = ((ConfigurationObject) lms).getMutableSet("RDSMeldungen");

			for (final TIC3Meldung meldung : bildeMeldungenAusDateien()) {
				meldungsmenge.add(meldung.getSystemObject());

				System.out.println(
						"Sende Daten für die Meldung '" + meldung + "', PID: " + meldung.getSystemObject().getPid());

				VersendeMeldungen.verbindung.subscribeSender(this, meldung.getSystemObject(), dd, SenderRole.source());

				final Data data = VersendeMeldungen.verbindung.createData(dd.getAttributeGroup());

				meldung.getMeldungsDaten().publizieren(data, null);
				final ResultData meldungsdaten = new ResultData(meldung.getSystemObject(), dd,
						System.currentTimeMillis(), data);
				VersendeMeldungen.verbindung.sendData(meldungsdaten);
			}

			// etwas warten, damit Quelle nicht verschwindet
			Thread.sleep(2000000);
		} catch (final Exception e) {
			e.printStackTrace();
			Assert.fail(e.getMessage());
		}
	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		// TODO Auto-generated method stub
		return false;
	}

	private TIC3Meldung neueMeldung(final TIC3MeldungsDaten meldungsdaten) throws TIC3Exception {
		final ConfigurationArea area = VersendeMeldungen.verbindung.getLocalConfigurationAuthority()
				.getConfigurationArea();

		final String pid = meldungsdaten.bildePid();
		SystemObject so = VersendeMeldungen.verbindung.getDataModel().getObject(pid);
		if (so == null) {
			try {
				so = area.createDynamicObject(
						(DynamicObjectType) VersendeMeldungen.verbindung.getDataModel().getType("typ.rdsMeldung"), pid,
						pid);
				System.out.println("Die Meldung " + pid + " wurde angelegt");
			} catch (final ConfigurationChangeException e) {
				throw new TIC3Exception(e.getMessage());
			}
		} else {
			System.out.println("Die Meldung " + pid + " existiert bereits");
		}

		final TIC3Meldung rdsmeldung = new TIC3Meldung(so, meldungsdaten);

		return rdsmeldung;
	}
}
