/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;

/**
 * Abstrakte Implementation eines Attributes eines TIC3-Elementes.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: AbstractTIC3InfoAttribut.java 63650 2016-09-22 17:11:32Z
 *          gieseler $
 */

public abstract class AbstractTIC3InfoAttribut implements TIC3InfoElement {

	/**
	 * Der zum Element geh&ouml;rende Name des XML-Attributes.
	 */
	private String xmlAttributName = "";

	/**
	 * Der Name des zum Element geh&ouml;renden DAV-Attributes.
	 */
	private String davAttributName = "";

	private String xmlValue;

	/**
	 * Gibt den Wert des XML-Attributes zur&uuml;ck.
	 * 
	 * @return Wert des XML-Attributes
	 */
	public String getXmlValue() {
		return xmlValue;
	}

	/**
	 * Setzt den Wert des XML-Attributes.
	 * 
	 * @param xmlValue
	 *            Wert des XML-Attributes
	 */
	public void setXmlValue(final String xmlValue) {
		this.xmlValue = xmlValue;
	}

	/**
	 * Erzeugt ein Element.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public AbstractTIC3InfoAttribut(final String xmlName, final String davName) {
		super();
		this.xmlAttributName = xmlName;
		this.davAttributName = davName;
	}

	// @Override
	// public void aktualisieren(final Data davdata) throws TIC3Exception {
	// getDavDaten(davdata);
	//
	// final Data ddata;
	// if ((getDavAttributName() != null) && (getDavAttributName().length() >
	// 0)) {
	// ddata = davdata.getItem(getDavAttributName());
	// } else {
	// ddata = davdata;
	// }
	// }

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		getXmlDaten(node);
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) throws TIC3Exception {
		final Attr attribute = doc.createAttribute(getXmlElementName());
		parent.appendChild(attribute);
		return null;
	}

	/**
	 * Gibt den zum Element geh&ouml;renden Namen des DAV-Attributes
	 * zur&uuml;ck.
	 *
	 * @return Name des DAV-Attributes
	 */
	protected String getDavAttributName() {
		return davAttributName;
	}

	@Override
	public void getDavDaten(final Data davdata) throws TIC3Exception {
		/** keine eigenen Daten */
	}

	@Override
	public void getXmlDaten(final Node node) throws TIC3Exception {
		final NamedNodeMap attributes = node.getAttributes();
		if (attributes == null) {
			return;
		}

		final Node namedItem = attributes.getNamedItem(xmlAttributName);

		if (namedItem == null) {
			return;
		}

		xmlValue = namedItem.getNodeValue();
	}

	@Override
	public String getXmlElementName() {
		return xmlAttributName;
	}

	/**
	 * Test, ob das Objekt einen Handler f&uuml;r einen XML-Dokument-Knoten
	 * besitzt.
	 *
	 * @param node
	 *            ein XML-Dokument-Knoten
	 * @return true, wenn das Objekt einen Handler f&uuml;r einen
	 *         XML-Dokument-Knoten besitzt, sonst false
	 */
	@Override
	public boolean hatNodeHandler(final Node node) {
		return node.getNodeName().equals(xmlAttributName);
	}

	@Override
	public void setDavDaten(final Data davdata) throws TIC3Exception {
		/** keine eigenen Daten */
	}

	@Override
	public void setXmlDaten(final Element element) throws TIC3Exception {
		/** keine eigenen Daten */
	}

	@Override
	public String toString() {
		return "XML-Attribut: " + getXmlElementName() + "=\"" + getXmlValue() + "\"";
	}
}
