/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;

/**
 * Repr&auml;sentiert einen optionalen Double-Wert.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id: TIC3OptionalerDoubleWert.java 63650 2016-09-22 17:11:32Z gieseler $
 */

public class TIC3OptionalerDoubleWert extends AbstractTIC3InfoElement {

	/**
	 * Wert.
	 */
	private Double wert;

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TIC3OptionalerDoubleWert(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	@Override
	public void getDavDaten(final Data davdata) throws TIC3Exception {
		if (davdata.isPlain()) {
			wert = new Double(davdata.asUnscaledValue().intValue());
		} else {
			wert = new Double(davdata.getUnscaledValue(getDavAttributName()).intValue());
		}
	}

	@Override
	public void getXmlDaten(final Node node) throws TIC3Exception {
		wert = Double.parseDouble(node.getTextContent());
	}

	@Override
	public void setDavDaten(final Data davdata) throws TIC3Exception {
		final double dwert;
		if (wert == null) {
			dwert = -1;
		} else {
			dwert = wert;
		}

		if (davdata.isPlain()) {
			davdata.asScaledValue().set(dwert);
		} else {
			if (dwert == -1) {
				davdata.getUnscaledValue(getDavAttributName()).set(dwert);
			} else {
				davdata.getScaledValue(getDavAttributName()).set(dwert);
			}
		}
	}

	@Override
	public void setXmlDaten(final Element element) {
		element.setTextContent(String.valueOf(wert));
	}

	/**
	 * Gibt den Wert zur&uuml;ck.
	 * 
	 * @return Wert
	 */
	public Double getWert() {
		return wert;
	}

	/**
	 * Setzt den Wert.
	 * 
	 * @param wert
	 *            Wert
	 */
	public void setWert(final double wert) {
		this.wert = new Double(wert);
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) throws TIC3Exception {
		if (wert == null) {
			return null;

		}

		return super.erzeugeDocumentElement(doc, parent);
	}

	@Override
	public String toString() {
		return getClass().getSimpleName() + " '" + getXmlElementName() + "': name='" + getNameAttribut().getXmlValue()
				+ "', Wert: '" + wert + "'";
	}
}
